@@ -1,4 +1,5 @@
 import os
+import socket
 import random
 import argparse
 import matplotlib
@@ -14,6 +15,19 @@ matplotlib.pyplot.rcParams['font.family'] = ['SimSong', 'Times New Roman']
 matplotlib.pyplot.rcParams['axes.unicode_minus'] = False
 
 
+def str2bool(string):
+    str2val = {"True": True, "False": False}
+    if string in str2val:
+        return str2val[string]
+    else:
+        raise ValueError(f"Expected one of {set(str2val.keys())}, got {string}")
+
+
+def is_port_in_use(port):
+    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
+        return s.connect_ex(('localhost', port)) == 0
+
+
 def password_generator(seed=51):   
     random.seed(seed)
     num_list = list(range(1, 10))
@@ -165,8 +179,10 @@ def get_args():
                         help="license sequence to call the openai api")
     parser.add_argument("--model", "-m", type=str, default="gpt-3.5-turbo", choices=["gpt-3.5-turbo", "gpt-4"],
                         help="name of the model interface provided by openai")
-    parser.add_argument("--trend", "-t", type=str, default="general", choices=["general", "poet", "tutor"],
+    parser.add_argument("--model_trend", "-mt", type=str, default="general", choices=["general", "poet", "tutor"],
                         help="set the response bias of the model")
+    parser.add_argument("--request_method", "-rm", type=str, default="post", choices=["official", "post"],
+                        help="set the method to request access to the api")
     parser.add_argument("--server_name", "-sn", type=str, default="127.0.0.1",
                         help="set the name of the server where the web page will run is usually done on the local machine")
     parser.add_argument("--server_port", "-sp", type=int, default=7860,
@@ -175,15 +191,15 @@ def get_args():
                         help="set for the traffic proxy server are the same as those for running web pages")
     parser.add_argument("--proxy_port", "-pp", type=int, default=7890,
                         help="set the port exposed by the traffic proxy server")
-    parser.add_argument("--share", "-s", type=bool, default=False,
+    parser.add_argument("--share", "-s", type=str2bool, default=False,
                         help="if True, will create a public network access url using gradio, but some localhost is not accessible (e.g. Google Colab)")
-    parser.add_argument("--debug", "-d", type=bool, default=False,
+    parser.add_argument("--debug", "-d", type=str2bool, default=False,
                         help="if True, will blocks the main thread from running and print the errors in output")
     # 可选参数
     parser.add_argument("--temperature", type=int, default=1)
     parser.add_argument("--top_p", type=int, default=1)
     parser.add_argument("--n", type=int, default=SESSIONNUM)
-    parser.add_argument("--stream", type=bool, default=False)
+    parser.add_argument("--stream", type=str2bool, default=False)
     parser.add_argument("--presence_penalty", type=int, default=0)
     parser.add_argument("--frequency_penalty", type=int, default=0)
     parser.add_argument("--logsdir", type=str, default="./logging")
@@ -193,12 +209,9 @@ def get_args():
 
 
 def main():
+    # 参数
     args = get_args()
     
-    # 代理
-    os.environ["http_proxy"] = f"http://{args.proxy_name}:{args.proxy_port}"
-    os.environ["https_proxy"] = f"http://{args.proxy_name}:{args.proxy_port}"
-    
     # 后端
     global gpt_model
     gpt_model = CallChatGPT(api_key=args.api_key,
@@ -209,14 +222,17 @@ def main():
                             stream=args.stream,
                             presence_penalty=args.presence_penalty,
                             frequency_penalty=args.frequency_penalty,
+                            proxy_name=args.proxy_name,
+                            proxy_port=args.proxy_port,
                             logsdir=args.logsdir,
                             logsname=args.logsname,
-                            trend=args.trend)
+                            model_trend=args.model_trend,
+                            request_method=args.request_method,)
 
     # 前端
     with gr.Blocks() as web:
         with gr.Box():
-            gr.Markdown("<center><h1>OpenAI</h1><h5>Welcome To Play - Code By HaoDaXia</h5></center>") 
+            gr.Markdown("<center><h1>OpenAI GPT</h1><h5>Welcome To Play - Code By HaoDaXia</h5></center>") 
             """控件显示"""
             # 一、聊天
             with gr.Tab(label="聊天", id=0):
@@ -335,6 +351,8 @@ def main():
                              outputs=[log_text_1st, in_text_3st, in_text_4st])
     
     # 启动
+    if is_port_in_use(args.server_port):
+        args.server_port += 1
     web.queue().launch(server_name=args.server_name,
                        server_port=args.server_port,
                        share=args.share,
